package Post2Buffer::App;

use strict;
use warnings;

use Data::Dumper;
use LWP::UserAgent;
use MT::Entry;
use URI;

use HTTP::Request::Common qw( POST );
use JSON qw( decode_json );
use MT::Util qw( encode_url );
use Post2Buffer::Util qw( doLog saveFile );

sub _hdlr_template_source_edit_entry {
    my ( $cb, $app, $tmpl_ref ) = @_;

    my $q       = $app->{'query'};
    my $blog_id = $q->param('blog_id');
    my $plugin  = MT->component('post2buffer');
    my $enable  = $plugin->get_config_value( 'BufferEnable', "blog:$blog_id" );

    ## case: disable
    return 1 unless $enable;

    ## case: enable
    my $old = <<EOF;
    <input type="text" name="revision-note" id="revision-note" class="text full" value="<mt:var name="revision-note" escape="html">" />
  </mtapp:setting>
</mt:if>
EOF

    $old = quotemeta($old);

    #    $old =~ s!(\\ )+!\\s+!g;

    my $new = $plugin->translate_templatized(<<'EOF');
<mtapp:Setting
  id="BufferMessage"
  label="<__trans phrase="Text (buffer)">"
  label_class="top-label"
  show_hint="0">
  <textarea name="BufferMessage" id="BufferMessage" class="text full low" cols="" rows=""><mt:Var name="BufferMessage" escape="html"></textarea>
</mtapp:Setting>
<mtapp:Setting
  id="BufferTrigger"
  label="<__trans phrase='Post (buffer)'>"
  label_class="top-label"
  show_hint="0">
  <select name="BufferTrigger" id="BufferTrigger">
    <option value="0"<mt:If name="BufferTrigger" eq="0"> selected="selected"</mt:If>><__trans phrase='None'></option>
    <option value="1"<mt:If name="BufferTrigger" eq="1"> selected="selected"</mt:If>><__trans phrase='Post now'></option>
    <option value="2"<mt:If name="BufferTrigger" eq="2"> selected="selected"</mt:If>><__trans phrase='Scheduled post'></option>
  </select>
</mtapp:Setting>
EOF

    $$tmpl_ref =~ s!($old)!$1$new!;
}

sub _hdlr_cms_post_save_entry {
    my ( $cb, $app, $obj ) = @_;

    my $q = $app->{'query'};
    return unless defined($q);

    my $blog_id = $q->param('blog_id');
    my $id      = $q->param('id');
    my $plugin  = MT->component('post2buffer');
    return unless defined($plugin);

    # case: disable
    my $enable = $plugin->get_config_value( 'BufferEnable', "blog:$blog_id" );
    return unless defined($enable);

    ## case: not published
    return unless $obj->status == MT::Entry::RELEASE();

    # case: invalid parameter
    my $entry = MT::Entry->load($id);
    return unless defined($entry);

    # case: enable
    my $message = $q->param('BufferMessage');
    my $option  = $q->param('BufferTrigger');
    return if !( $option == 1 || $option == 2 );

    my $access_token = $plugin->get_config_value( 'BufferAccessToken',    'system' );
    my $shorten      = $plugin->get_config_value( 'BufferLinkShortening', 'system' );
    return unless defined($access_token);

    # get profile information
    my $ua = LWP::UserAgent->new;
    $ua->timeout(30);
    $ua->agent('Mozilla');
    $ua->ssl_opts( verify_hostname => 0 );

    # case: error occurred (profile)
    my $res = $ua->get( 'https://api.bufferapp.com/1/profiles.json?access_token=' . encode_url($access_token) );
    return unless $res->is_success;

    my $items;
    my @service;
    my @id;
    my $i = 0;

    my $json = $res->content;
    if ( $json && !( $json =~ /^null$/i ) ) {
        $items = JSON::decode_json($json);
        for my $item (@$items) {
            $service[$i] = $item->{'formatted_service'};
            $id[$i]      = $item->{'id'};
            $i++;
        }
    }
    return unless defined($items);

    my $postText = $message ? $message : $entry->title;
    my $schedule = 0;
    $schedule = 1 if ( $option == 1 );

    my $bufferParams = {
        access_token    => $access_token,
        text            => $postText . " " . $entry->permalink,
        now             => $schedule,
        shorten         => $shorten ? 1 : 0,
        'profile_ids[]' => [@id],
        'media[title]'  => $entry->title,
        'media[link]'   => $entry->permalink,
    };

    my $req = POST( 'https://api.bufferapp.com/1/updates/create.json', [$bufferParams] );
    $res = $ua->request($req);
    if ( $res->is_success ) {
        $app->log(
            {   message => $plugin->translate( 'Post to buffer is completed. [_1] (ID: [_2])', $entry->title, $entry->id ),
                level   => MT::Log::INFO(),
                blog_id => $blog_id,
            }
        );
    }
    else {
        $app->log(
            {   message  => $plugin->translate( 'Error occurred : post to buffer [_1] (ID: [_2])', $entry->title, $entry->id ),
                metadata => $entry->title,
                level    => MT::Log::ERRRO(),
                blog_id  => $blog_id,
            }
        );
    }

    1;
}

1;
